using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Macke.KonfigurationNS
{
    /// <summary>
    /// Formular zur einfachen nderung der Konfiguration.
    /// </summary>
    public partial class FormKonfiguration : Form
    {
        /// <summary>
        /// Referenz auf das Konfiguration-Objekt.
        /// </summary>
        private Konfiguration k;
        
        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="k">Zu verwendende Konfiguration.</param>
        public FormKonfiguration(Konfiguration k)
        {
            this.k = k;
            InitializeComponent();
        }

        /// <summary>
        /// Laden des Formulars.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormKonfiguration_Load(object sender, EventArgs e)
        {
            try
            {
                // Liste der Bereiche unterhalb des Wurzelelements erstellen
                List<Konfiguration.Bereich> bereiche = this.k.GetBereiche();
                // TreeNode fr jeden Bereich erstellen
                foreach (Konfiguration.Bereich bereich in bereiche)
                {
                    TreeNode n = this.treeViewBereiche.Nodes.Add(bereich.Pfad, bereich.Bezeichnung);
                    // Unterbereiche ermitteln und hinzufgen
                    this.SetzeUnterbereicheZuBereich(n);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Fehler");
                this.Close();
            }
        }

        /// <summary>
        /// Nach dem Auswhlen eines TreeNodes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeViewBereiche_AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                // das Formular mit den entsprechenden Einstellungen fllen
                this.SetzeEinstellungenZuBereich(e.Node);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Fehler");
            }
        }

        /// <summary>
        /// Zeigt die Einstellungen zum bergebenen Bereich an.
        /// </summary>
        /// <param name="n">TreeNode des aktuellen Bereichs.</param>
        private void SetzeEinstellungenZuBereich(TreeNode n)
        {
            Konfiguration.Bereich bereich = this.k.GetBereich(n.Name);
            
            // alte Controls raus...
            int zaehler = this.panelEinstellungen.Controls.Count;
            for (int i = 0; i < zaehler; i++)
            {
                this.panelEinstellungen.Controls.RemoveAt(0);
            }
            this.toolTipBeschreibung.RemoveAll();

            // ...neue rein
            zaehler = 0;
            foreach (Konfiguration.Einstellung einstellung in bereich.Einstellungen)
            {
                zaehler++;

                Panel p = new Panel();
                Label lb = new Label();
                Label ls = new Label();
                TextBox tb = new TextBox();

                // Panel
                p.Name = "panel" + zaehler;
                p.Location = new Point(5, 5 + (zaehler - 1) * 80);
                p.Size = new Size(this.panelEinstellungen.Width - 15, 80);
                p.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right | System.Windows.Forms.AnchorStyles.Top)));

                // Label Bezeichnung
                lb.Name = "labelBez" + zaehler;
                lb.Text = einstellung.Bezeichnung;
                lb.Location = new Point(0, 0);
                lb.Size = new Size(p.Width, 15);
                lb.Tag = einstellung.Beschreibung;
                this.toolTipBeschreibung.SetToolTip(lb, einstellung.Beschreibung);
                p.Controls.Add(lb);

                // Label Standard
                ls.Name = "labelStd" + zaehler;
                ls.Text = einstellung.StandardWert;
                ls.Location = new Point(5, lb.Height);
                ls.Size = new Size(p.Width, 20);
                ls.Font = new Font("Microsoft Sans Serif", 7F, System.Drawing.FontStyle.Italic);
                p.Controls.Add(ls);

                // Textbox
                tb.Name = einstellung.Pfad;
                tb.Text = einstellung.EigenerWert;
                tb.Location = new Point(0, lb.Height + ls.Height);
                tb.Size = new Size(p.Width, 20);
                tb.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
                tb.TabIndex = zaehler;
                tb.TextChanged += new System.EventHandler(this.Einstellung_Speichern);
                tb.Leave += new System.EventHandler(this.Einstellung_Speichern);
                p.Controls.Add(tb);

                // zum Formular hinzufgen
                this.panelEinstellungen.Controls.Add(p);
            }
        }

        /// <summary>
        /// Fgt die Unterbereiche zum bergebenen Bereich hinzu.
        /// </summary>
        /// <param name="n">TreeNode des aktuellen Bereichs.</param>
        private void SetzeUnterbereicheZuBereich(TreeNode n)
        {
            // Unterbereiche ermitteln
            List<Konfiguration.Bereich> unterbereiche = this.k.GetBereiche(n.Name);

            foreach (Konfiguration.Bereich unterbereich in unterbereiche)
            {
                TreeNode unternode = n.Nodes.Add(unterbereich.Pfad,unterbereich.Bezeichnung);
                // Rekursion fr weitere Unterbereiche
                if (this.k.GetBereiche(unternode.Name).Count > 0)
                {
                    this.SetzeUnterbereicheZuBereich(unternode);
                }
            }

        }
        
        /// <summary>
        /// Klick des Buttons "Abbrechen".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAbbrechen_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Setzen einer Einstellung nachdem diese gendert wurde.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Einstellung_Speichern(object sender, EventArgs e)
        {
            if (sender is TextBox)
            {
                TextBox tb = sender as TextBox;
                try
                {
                    this.k.SetEinstellung(tb.Name, tb.Text);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Fehler");
                }
            }
            this.buttonSpeichern.Enabled = true;
        }

        /// <summary>
        /// Klick des Buttons "Speichern".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSpeichern_Click(object sender, EventArgs e)
        {
            this.buttonSpeichern.Enabled = false;
            try
            {
                this.k.Speichern();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Fehler");
                this.buttonSpeichern.Enabled = true;
            }
        }
    }
}